#include "controller/controller.h"
#include "$(appName)-ref.h"
#include "view/main_window.h"
#include "model/settings.h"
#include <device/callback.h>
#include <device/display.h>

static void __wallpaper_changed_cb(system_settings_key_e key, void *user_data);
static void __device_cb(device_callback_e type, void *value, void *user_data);

static void __set_current_lock(void);
static void __swipe_unlocked_cb(void);

bool
start_application(void)
{
	if (!create_main_window()) {
		dlog_print(DLOG_ERROR, LOG_TAG, "View creation failed: on window handle");
		destroy_main_window();
		return false;
	}

	__set_current_lock();
	add_main_window_callbacks(__swipe_unlocked_cb);
	__wallpaper_changed_cb(SYSTEM_SETTINGS_KEY_WALLPAPER_LOCK_SCREEN, NULL);
	system_settings_set_changed_cb(SYSTEM_SETTINGS_KEY_WALLPAPER_LOCK_SCREEN, __wallpaper_changed_cb, NULL);
	device_add_callback(DEVICE_CALLBACK_DISPLAY_STATE, __device_cb, NULL);

	return true;
}

void
terminate_application(void)
{
	del_main_window_callbacks();
	destroy_main_window();
}

static void
__wallpaper_changed_cb(system_settings_key_e key, void *user_data)
{
	char *wallpaper_file_name = NULL;
	int res = system_settings_get_value_string(SYSTEM_SETTINGS_KEY_WALLPAPER_LOCK_SCREEN, &wallpaper_file_name);

	if (res == SYSTEM_SETTINGS_ERROR_NONE)
		set_main_window_current_wallpaper(wallpaper_file_name);
	else
		dlog_print(DLOG_ERROR, LOG_TAG, "_wallpaper_changed_cb error");

	free(wallpaper_file_name);
}

static void
__set_current_lock(void)
{
	switch (get_lock_type()) {
		case LOCKSCREEN_NONE:
			deactivate_main_window_swipe_lock();
			break;
		case LOCKSCREEN_SWIPE:
			activate_main_window_swipe_lock();
			break;
		default:
			break;
	}
}

static void
__device_cb(device_callback_e type, void *value, void *user_data)
{
	switch ((display_state_e)value) {
		case DISPLAY_STATE_NORMAL:
			__set_current_lock();
			break;
		case DISPLAY_STATE_SCREEN_DIM:
			break;
		case DISPLAY_STATE_SCREEN_OFF:
			break;
		default:
			__set_current_lock();
			break;
	}
}

static void
__swipe_unlocked_cb(void)
{
	deactivate_main_window_swipe_lock();
}
